@@ -1,9 +1,10 @@
-# SPDX-License-Identifer: Apache-2.0
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2021 The Meson development team
 from __future__ import annotations
 
 from .common import cmake_is_debug
 from .. import mlog
+from ..mesonlib import Version
 
 from pathlib import Path
 import re
@@ -15,6 +16,28 @@ if T.TYPE_CHECKING:
     from ..compilers import Compiler
     from ..dependencies import MissingCompiler
 
+# Small duplication of ExtraFramework to parse full
+# framework paths as exposed by CMake
+def _get_framework_latest_version(path: Path) -> str:
+    versions: list[Version] = []
+    for each in path.glob('Versions/*'):
+        # macOS filesystems are usually case-insensitive
+        if each.name.lower() == 'current':
+            continue
+        versions.append(Version(each.name))
+    if len(versions) == 0:
+        # most system frameworks do not have a 'Versions' directory
+        return 'Headers'
+    return 'Versions/{}/Headers'.format(sorted(versions)[-1]._s)
+
+def _get_framework_include_path(path: Path) -> T.Optional[str]:
+    trials = ('Headers', 'Versions/Current/Headers', _get_framework_latest_version(path))
+    for each in trials:
+        trial = path / each
+        if trial.is_dir():
+            return trial.as_posix()
+    return None
+
 class ResolvedTarget:
     def __init__(self) -> None:
         self.include_directories: T.List[str] = []
@@ -46,16 +69,35 @@ def resolve_cmake_trace_targets(target_name: str,
             continue
 
         if curr not in trace.targets:
+            curr_path = Path(curr)
             if reg_is_lib.match(curr):
                 res.libraries += [curr]
-            elif Path(curr).is_absolute() and Path(curr).exists():
-                res.libraries += [curr]
-            elif env.machines.build.is_windows() and reg_is_maybe_bare_lib.match(curr) and clib_compiler:
-                # On Windows, CMake library dependencies can be passed as bare library names,
+            elif curr_path.is_absolute() and curr_path.exists():
+                if any(x.endswith('.framework') for x in curr_path.parts):
+                    # Frameworks detected by CMake are passed as absolute paths
+                    # Split into -F/path/to/ and -framework name
+                    path_to_framework = []
+                    # Try to slice off the `Versions/X/name.tbd`
+                    for x in curr_path.parts:
+                        path_to_framework.append(x)
+                        if x.endswith('.framework'):
+                            break
+                    curr_path = Path(*path_to_framework)
+                    framework_path = curr_path.parent
+                    framework_name = curr_path.stem
+                    res.libraries += [f'-F{framework_path}', '-framework', framework_name]
+                else:
+                    res.libraries += [curr]
+            elif reg_is_maybe_bare_lib.match(curr) and clib_compiler:
+                # CMake library dependencies can be passed as bare library names,
                 # CMake brute-forces a combination of prefix/suffix combinations to find the
                 # right library. Assume any bare argument passed which is not also a CMake
                 # target must be a system library we should try to link against.
-                res.libraries += clib_compiler.find_library(curr, env, [])
+                flib = clib_compiler.find_library(curr, env, [])
+                if flib is not None:
+                    res.libraries += flib
+                else:
+                    not_found_warning(curr)
             else:
                 not_found_warning(curr)
             continue
@@ -95,9 +137,9 @@ def resolve_cmake_trace_targets(target_name: str,
         elif 'IMPORTED_IMPLIB' in tgt.properties:
             res.libraries += [x for x in tgt.properties['IMPORTED_IMPLIB'] if x]
         elif f'IMPORTED_LOCATION_{cfg}' in tgt.properties:
-            res.libraries += [x for x in tgt.properties[f'IMPORTED_LOCATION_{cfg}'] if x]
+            targets += [x for x in tgt.properties[f'IMPORTED_LOCATION_{cfg}'] if x]
         elif 'IMPORTED_LOCATION' in tgt.properties:
-            res.libraries += [x for x in tgt.properties['IMPORTED_LOCATION'] if x]
+            targets += [x for x in tgt.properties['IMPORTED_LOCATION'] if x]
 
         if 'LINK_LIBRARIES' in tgt.properties:
             targets += [x for x in tgt.properties['LINK_LIBRARIES'] if x]
@@ -111,9 +153,9 @@ def resolve_cmake_trace_targets(target_name: str,
 
         processed_targets += [curr]
 
-    res.include_directories = sorted(set(res.include_directories))
-    res.link_flags = sorted(set(res.link_flags))
-    res.public_compile_opts = sorted(set(res.public_compile_opts))
-    res.libraries = sorted(set(res.libraries))
+    # Do not sort flags here -- this breaks
+    # semantics of eg. `-framework CoreAudio`
+    # or `-Lpath/to/root -llibrary`
+    # see eg. #11113
 
     return res
