@@ -1,16 +1,6 @@
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2012-2022 The Meson development team
 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-
-#     http://www.apache.org/licenses/LICENSE-2.0
-
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
 from __future__ import annotations
 
 import os.path
@@ -19,7 +9,6 @@ import subprocess
 import typing as T
 
 from .. import mesonlib
-from .. import mlog
 from ..arglist import CompilerArgs
 from ..linkers import RSPFileSyntax
 from ..mesonlib import (
@@ -28,9 +17,6 @@ from ..mesonlib import (
 
 from . import compilers
 from .compilers import (
-    d_dmd_buildtype_args,
-    d_gdc_buildtype_args,
-    d_ldc_buildtype_args,
     clike_debug_args,
     Compiler,
     CompileCheckMode,
@@ -39,51 +25,67 @@ from .mixins.gnu import GnuCompiler
 from .mixins.gnu import gnu_common_warning_args
 
 if T.TYPE_CHECKING:
+    from ..build import DFeatures
     from ..dependencies import Dependency
-    from ..programs import ExternalProgram
     from ..envconfig import MachineInfo
     from ..environment import Environment
-    from ..linkers import DynamicLinker
+    from ..linkers.linkers import DynamicLinker
     from ..mesonlib import MachineChoice
 
     CompilerMixinBase = Compiler
 else:
     CompilerMixinBase = object
 
-d_feature_args = {'gcc':  {'unittest': '-funittest',
-                           'debug': '-fdebug',
-                           'version': '-fversion',
-                           'import_dir': '-J'
-                           },
-                  'llvm': {'unittest': '-unittest',
-                           'debug': '-d-debug',
-                           'version': '-d-version',
-                           'import_dir': '-J'
-                           },
-                  'dmd':  {'unittest': '-unittest',
-                           'debug': '-debug',
-                           'version': '-version',
-                           'import_dir': '-J'
-                           }
-                  }  # type: T.Dict[str, T.Dict[str, str]]
+d_feature_args: T.Dict[str, T.Dict[str, str]] = {
+    'gcc':  {
+        'unittest': '-funittest',
+        'debug': '-fdebug',
+        'version': '-fversion',
+        'import_dir': '-J'
+    },
+    'llvm': {
+        'unittest': '-unittest',
+        'debug': '-d-debug',
+        'version': '-d-version',
+        'import_dir': '-J'
+    },
+    'dmd':  {
+        'unittest': '-unittest',
+        'debug': '-debug',
+        'version': '-version',
+        'import_dir': '-J'
+    }
+}
 
-ldc_optimization_args = {'plain': [],
-                         '0': [],
-                         'g': [],
-                         '1': ['-O1'],
-                         '2': ['-O2'],
-                         '3': ['-O3'],
-                         's': ['-Oz'],
-                         }  # type: T.Dict[str, T.List[str]]
+ldc_optimization_args: T.Dict[str, T.List[str]] = {
+    'plain': [],
+    '0': [],
+    'g': [],
+    '1': ['-O1'],
+    '2': ['-O2', '-enable-inlining', '-Hkeep-all-bodies'],
+    '3': ['-O3', '-enable-inlining', '-Hkeep-all-bodies'],
+    's': ['-Oz'],
+}
 
-dmd_optimization_args = {'plain': [],
-                         '0': [],
-                         'g': [],
-                         '1': ['-O'],
-                         '2': ['-O'],
-                         '3': ['-O'],
-                         's': ['-O'],
-                         }  # type: T.Dict[str, T.List[str]]
+dmd_optimization_args: T.Dict[str, T.List[str]] = {
+    'plain': [],
+    '0': [],
+    'g': [],
+    '1': ['-O'],
+    '2': ['-O', '-inline'],
+    '3': ['-O', '-inline'],
+    's': ['-O'],
+}
+
+gdc_optimization_args: T.Dict[str, T.List[str]] = {
+    'plain': [],
+    '0': ['-O0'],
+    'g': ['-Og'],
+    '1': ['-O1'],
+    '2': ['-O2', '-finline-functions'],
+    '3': ['-O3', '-finline-functions'],
+    's': ['-Os'],
+}
 
 
 class DmdLikeCompilerMixin(CompilerMixinBase):
@@ -102,7 +104,7 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
             self._dmd_has_depfile = version_compare(dmd_frontend_version, ">=2.095.0")
 
     if T.TYPE_CHECKING:
-        mscrt_args = {}  # type: T.Dict[str, T.List[str]]
+        mscrt_args: T.Dict[str, T.List[str]] = {}
 
         def _get_target_arch_args(self) -> T.List[str]: ...
 
@@ -164,96 +166,8 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
             return []
         return ['-fPIC']
 
-    def get_feature_args(self, kwargs: T.Dict[str, T.Any], build_to_src: str) -> T.List[str]:
-        # TODO: using a TypeDict here would improve this
-        res = []
-        # get_feature_args can be called multiple times for the same target when there is generated source
-        # so we have to copy the kwargs (target.d_features) dict before popping from it
-        kwargs = kwargs.copy()
-        if 'unittest' in kwargs:
-            unittest = kwargs.pop('unittest')
-            unittest_arg = d_feature_args[self.id]['unittest']
-            if not unittest_arg:
-                raise EnvironmentException('D compiler %s does not support the "unittest" feature.' % self.name_string())
-            if unittest:
-                res.append(unittest_arg)
-
-        if 'debug' in kwargs:
-            debug_level = -1
-            debugs = kwargs.pop('debug')
-            if not isinstance(debugs, list):
-                debugs = [debugs]
-
-            debug_arg = d_feature_args[self.id]['debug']
-            if not debug_arg:
-                raise EnvironmentException('D compiler %s does not support conditional debug identifiers.' % self.name_string())
-
-            # Parse all debug identifiers and the largest debug level identifier
-            for d in debugs:
-                if isinstance(d, int):
-                    if d > debug_level:
-                        debug_level = d
-                elif isinstance(d, str) and d.isdigit():
-                    if int(d) > debug_level:
-                        debug_level = int(d)
-                else:
-                    res.append(f'{debug_arg}={d}')
-
-            if debug_level >= 0:
-                res.append(f'{debug_arg}={debug_level}')
-
-        if 'versions' in kwargs:
-            version_level = -1
-            versions = kwargs.pop('versions')
-            if not isinstance(versions, list):
-                versions = [versions]
-
-            version_arg = d_feature_args[self.id]['version']
-            if not version_arg:
-                raise EnvironmentException('D compiler %s does not support conditional version identifiers.' % self.name_string())
-
-            # Parse all version identifiers and the largest version level identifier
-            for v in versions:
-                if isinstance(v, int):
-                    if v > version_level:
-                        version_level = v
-                elif isinstance(v, str) and v.isdigit():
-                    if int(v) > version_level:
-                        version_level = int(v)
-                else:
-                    res.append(f'{version_arg}={v}')
-
-            if version_level >= 0:
-                res.append(f'{version_arg}={version_level}')
-
-        if 'import_dirs' in kwargs:
-            import_dirs = kwargs.pop('import_dirs')
-            if not isinstance(import_dirs, list):
-                import_dirs = [import_dirs]
-
-            import_dir_arg = d_feature_args[self.id]['import_dir']
-            if not import_dir_arg:
-                raise EnvironmentException('D compiler %s does not support the "string import directories" feature.' % self.name_string())
-            for idir_obj in import_dirs:
-                basedir = idir_obj.get_curdir()
-                for idir in idir_obj.get_incdirs():
-                    bldtreedir = os.path.join(basedir, idir)
-                    # Avoid superfluous '/.' at the end of paths when d is '.'
-                    if idir not in ('', '.'):
-                        expdir = bldtreedir
-                    else:
-                        expdir = basedir
-                    srctreedir = os.path.join(build_to_src, expdir)
-                    res.append(f'{import_dir_arg}{srctreedir}')
-                    res.append(f'{import_dir_arg}{bldtreedir}')
-
-        if kwargs:
-            raise EnvironmentException('Unknown D compiler feature(s) selected: %s' % ', '.join(kwargs.keys()))
-
-        return res
-
-    def get_buildtype_linker_args(self, buildtype: str) -> T.List[str]:
-        if buildtype != 'plain':
+    def get_optimization_link_args(self, optimization_level: str) -> T.List[str]:
+        if optimization_level != 'plain':
             return self._get_target_arch_args()
         return []
 
@@ -271,7 +185,7 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
             # The way that dmd and ldc pass rpath to gcc is different than we would
             # do directly, each argument -rpath and the value to rpath, need to be
             # split into two separate arguments both prefaced with the -L=.
-            args = []
+            args: T.List[str] = []
             (rpath_args, rpath_dirs_to_remove) = super().build_rpath_args(
                     env, build_dir, from_dir, rpath_paths, build_rpath, install_rpath)
             for r in rpath_args:
@@ -292,7 +206,7 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
         # can understand.
         # The flags might have been added by pkg-config files,
         # and are therefore out of the user's control.
-        dcargs = []
+        dcargs: T.List[str] = []
         # whether we hit a linker argument that expect another arg
         # see the comment in the "-L" section
         link_expect_arg = False
@@ -301,7 +215,7 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
         ]
         for arg in args:
             # Translate OS specific arguments first.
-            osargs = []  # type: T.List[str]
+            osargs: T.List[str] = []
             if info.is_windows():
                 osargs = cls.translate_arg_to_windows(arg)
             elif info.is_darwin():
@@ -415,7 +329,7 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
 
     @classmethod
     def translate_arg_to_windows(cls, arg: str) -> T.List[str]:
-        args = []
+        args: T.List[str] = []
         if arg.startswith('-Wl,'):
             # Translate linker arguments here.
             linkargs = arg[arg.index(',') + 1:].split(',')
@@ -441,7 +355,7 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
 
     @classmethod
     def _translate_arg_to_osx(cls, arg: str) -> T.List[str]:
-        args = []
+        args: T.List[str] = []
         if arg.startswith('-install_name'):
             args.append('-L=' + arg)
         return args
@@ -460,31 +374,7 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
     def _get_crt_args(self, crt_val: str, buildtype: str) -> T.List[str]:
         if not self.info.is_windows():
             return []
-
-        if crt_val in self.mscrt_args:
-            return self.mscrt_args[crt_val]
-        assert crt_val in {'from_buildtype', 'static_from_buildtype'}
-
-        dbg = 'mdd'
-        rel = 'md'
-        if crt_val == 'static_from_buildtype':
-            dbg = 'mtd'
-            rel = 'mt'
-
-        # Match what build type flags used to do.
-        if buildtype == 'plain':
-            return []
-        elif buildtype == 'debug':
-            return self.mscrt_args[dbg]
-        elif buildtype == 'debugoptimized':
-            return self.mscrt_args[rel]
-        elif buildtype == 'release':
-            return self.mscrt_args[rel]
-        elif buildtype == 'minsize':
-            return self.mscrt_args[rel]
-        else:
-            assert buildtype == 'custom'
-            raise EnvironmentException('Requested C runtime based on buildtype, but buildtype is "custom".')
+        return self.mscrt_args[self.get_crt_val(crt_val, buildtype)]
 
     def get_soname_args(self, env: 'Environment', prefix: str, shlib_name: str,
                         suffix: str, soversion: str,
@@ -494,15 +384,14 @@ class DmdLikeCompilerMixin(CompilerMixinBase):
 
         # LDC and DMD actually do use a linker, but they proxy all of that with
         # their own arguments
+        soargs: T.List[str] = []
         if self.linker.id.startswith('ld.'):
-            soargs = []
             for arg in sargs:
                 a, b = arg.split(',', maxsplit=1)
                 soargs.append(a)
                 soargs.append(self.LINKER_PREFIX + b)
             return soargs
         elif self.linker.id.startswith('ld64'):
-            soargs = []
             for arg in sargs:
                 if not arg.startswith(self.LINKER_PREFIX):
                     soargs.append(self.LINKER_PREFIX + arg)
@@ -542,29 +431,35 @@ class DCompiler(Compiler):
 
     def __init__(self, exelist: T.List[str], version: str, for_machine: MachineChoice,
                  info: 'MachineInfo', arch: str, *,
-                 exe_wrapper: T.Optional['ExternalProgram'] = None,
                  linker: T.Optional['DynamicLinker'] = None,
                  full_version: T.Optional[str] = None,
                  is_cross: bool = False):
         super().__init__([], exelist, version, for_machine, info, linker=linker,
                          full_version=full_version, is_cross=is_cross)
         self.arch = arch
-        self.exe_wrapper = exe_wrapper
 
     def sanity_check(self, work_dir: str, environment: 'Environment') -> None:
         source_name = os.path.join(work_dir, 'sanity.d')
         output_name = os.path.join(work_dir, 'dtest')
         with open(source_name, 'w', encoding='utf-8') as ofile:
             ofile.write('''void main() { }''')
-        pc = subprocess.Popen(self.exelist + self.get_output_args(output_name) + self._get_target_arch_args() + [source_name], cwd=work_dir)
+
+        compile_cmdlist = self.exelist + self.get_output_args(output_name) + self._get_target_arch_args() + [source_name]
+
+        # If cross-compiling, we can't run the sanity check, only compile it.
+        if self.is_cross and not environment.has_exe_wrapper():
+            compile_cmdlist += self.get_compile_only_args()
+
+        pc = subprocess.Popen(compile_cmdlist, cwd=work_dir)
         pc.wait()
         if pc.returncode != 0:
-            raise EnvironmentException('D compiler %s can not compile programs.' % self.name_string())
+            raise EnvironmentException('D compiler %s cannot compile programs.' % self.name_string())
+
         if self.is_cross:
-            if self.exe_wrapper is None:
+            if not environment.has_exe_wrapper():
                 # Can't check if the binaries run so we have to assume they do
                 return
-            cmdlist = self.exe_wrapper.get_command() + [output_name]
+            cmdlist = environment.exe_wrapper.get_command() + [output_name]
         else:
             cmdlist = [output_name]
         if subprocess.call(cmdlist) != 0:
@@ -581,96 +476,70 @@ class DCompiler(Compiler):
             return []
         return ['-fPIC']
 
-    def get_feature_args(self, kwargs: T.Dict[str, T.Any], build_to_src: str) -> T.List[str]:
-        # TODO: using a TypeDict here would improve this
-        res = []
-        # get_feature_args can be called multiple times for the same target when there is generated source
-        # so we have to copy the kwargs (target.d_features) dict before popping from it
-        kwargs = kwargs.copy()
-        if 'unittest' in kwargs:
-            unittest = kwargs.pop('unittest')
-            unittest_arg = d_feature_args[self.id]['unittest']
-            if not unittest_arg:
-                raise EnvironmentException('D compiler %s does not support the "unittest" feature.' % self.name_string())
-            if unittest:
-                res.append(unittest_arg)
+    def get_feature_args(self, kwargs: DFeatures, build_to_src: str) -> T.List[str]:
+        res: T.List[str] = []
+        unittest_arg = d_feature_args[self.id]['unittest']
+        if not unittest_arg:
+            raise EnvironmentException('D compiler %s does not support the "unittest" feature.' % self.name_string())
+        if kwargs['unittest']:
+            res.append(unittest_arg)
 
-        if 'debug' in kwargs:
-            debug_level = -1
-            debugs = kwargs.pop('debug')
-            if not isinstance(debugs, list):
-                debugs = [debugs]
+        debug_level = -1
+        debug_arg = d_feature_args[self.id]['debug']
+        if not debug_arg:
+            raise EnvironmentException('D compiler %s does not support conditional debug identifiers.' % self.name_string())
 
-            debug_arg = d_feature_args[self.id]['debug']
-            if not debug_arg:
-                raise EnvironmentException('D compiler %s does not support conditional debug identifiers.' % self.name_string())
+        # Parse all debug identifiers and the largest debug level identifier
+        for d in kwargs['debug']:
+            if isinstance(d, int):
+                debug_level = max(debug_level, d)
+            elif isinstance(d, str) and d.isdigit():
+                debug_level = max(debug_level, int(d))
+            else:
+                res.append(f'{debug_arg}={d}')
 
-            # Parse all debug identifiers and the largest debug level identifier
-            for d in debugs:
-                if isinstance(d, int):
-                    if d > debug_level:
-                        debug_level = d
-                elif isinstance(d, str) and d.isdigit():
-                    if int(d) > debug_level:
-                        debug_level = int(d)
+        if debug_level >= 0:
+            res.append(f'{debug_arg}={debug_level}')
+
+        version_level = -1
+        version_arg = d_feature_args[self.id]['version']
+        if not version_arg:
+            raise EnvironmentException('D compiler %s does not support conditional version identifiers.' % self.name_string())
+
+        # Parse all version identifiers and the largest version level identifier
+        for v in kwargs['versions']:
+            if isinstance(v, int):
+                version_level = max(version_level, v)
+            elif isinstance(v, str) and v.isdigit():
+                version_level = max(version_level, int(v))
+            else:
+                res.append(f'{version_arg}={v}')
+
+        if version_level >= 0:
+            res.append(f'{version_arg}={version_level}')
+
+        import_dir_arg = d_feature_args[self.id]['import_dir']
+        if not import_dir_arg:
+            raise EnvironmentException('D compiler %s does not support the "string import directories" feature.' % self.name_string())
+        # TODO: ImportDirs.to_string_list(), but we need both the project source
+        # root and project build root for that.
+        for idir_obj in kwargs['import_dirs']:
+            basedir = idir_obj.get_curdir()
+            for idir in idir_obj.get_incdirs():
+                bldtreedir = os.path.join(basedir, idir)
+                # Avoid superfluous '/.' at the end of paths when d is '.'
+                if idir not in ('', '.'):
+                    expdir = bldtreedir
                 else:
-                    res.append(f'{debug_arg}={d}')
-
-            if debug_level >= 0:
-                res.append(f'{debug_arg}={debug_level}')
-
-        if 'versions' in kwargs:
-            version_level = -1
-            versions = kwargs.pop('versions')
-            if not isinstance(versions, list):
-                versions = [versions]
-
-            version_arg = d_feature_args[self.id]['version']
-            if not version_arg:
-                raise EnvironmentException('D compiler %s does not support conditional version identifiers.' % self.name_string())
-
-            # Parse all version identifiers and the largest version level identifier
-            for v in versions:
-                if isinstance(v, int):
-                    if v > version_level:
-                        version_level = v
-                elif isinstance(v, str) and v.isdigit():
-                    if int(v) > version_level:
-                        version_level = int(v)
-                else:
-                    res.append(f'{version_arg}={v}')
-
-            if version_level >= 0:
-                res.append(f'{version_arg}={version_level}')
-
-        if 'import_dirs' in kwargs:
-            import_dirs = kwargs.pop('import_dirs')
-            if not isinstance(import_dirs, list):
-                import_dirs = [import_dirs]
-
-            import_dir_arg = d_feature_args[self.id]['import_dir']
-            if not import_dir_arg:
-                raise EnvironmentException('D compiler %s does not support the "string import directories" feature.' % self.name_string())
-            for idir_obj in import_dirs:
-                basedir = idir_obj.get_curdir()
-                for idir in idir_obj.get_incdirs():
-                    bldtreedir = os.path.join(basedir, idir)
-                    # Avoid superfluous '/.' at the end of paths when d is '.'
-                    if idir not in ('', '.'):
-                        expdir = bldtreedir
-                    else:
-                        expdir = basedir
-                    srctreedir = os.path.join(build_to_src, expdir)
-                    res.append(f'{import_dir_arg}{srctreedir}')
-                    res.append(f'{import_dir_arg}{bldtreedir}')
-
-        if kwargs:
-            raise EnvironmentException('Unknown D compiler feature(s) selected: %s' % ', '.join(kwargs.keys()))
+                    expdir = basedir
+                srctreedir = os.path.join(build_to_src, expdir)
+                res.append(f'{import_dir_arg}{srctreedir}')
+                res.append(f'{import_dir_arg}{bldtreedir}')
 
         return res
 
-    def get_buildtype_linker_args(self, buildtype: str) -> T.List[str]:
-        if buildtype != 'plain':
+    def get_optimization_link_args(self, optimization_level: str) -> T.List[str]:
+        if optimization_level != 'plain':
             return self._get_target_arch_args()
         return []
 
@@ -684,7 +553,9 @@ class DCompiler(Compiler):
         # LDC2 on Windows targets to current OS architecture, but
         # it should follow the target specified by the MSVC toolchain.
         if self.info.is_windows():
-            if self.arch == 'x86_64':
+            if self.is_cross:
+                return [f'-mtriple={self.arch}-windows-msvc']
+            elif self.arch == 'x86_64':
                 return ['-m64']
             return ['-m32']
         return []
@@ -706,36 +577,17 @@ class DCompiler(Compiler):
                 args.append(extra_args)
         return args
 
-    def run(self, code: 'mesonlib.FileOrString', env: 'Environment', *,
+    def run(self, code: 'mesonlib.FileOrString', env: 'Environment',
             extra_args: T.Union[T.List[str], T.Callable[[CompileCheckMode], T.List[str]], None] = None,
-            dependencies: T.Optional[T.List['Dependency']] = None) -> compilers.RunResult:
-        need_exe_wrapper = env.need_exe_wrapper(self.for_machine)
-        if need_exe_wrapper and self.exe_wrapper is None:
-            raise compilers.CrossNoRunException('Can not run test applications in this cross environment.')
+            dependencies: T.Optional[T.List['Dependency']] = None,
+            run_env: T.Optional[T.Dict[str, str]] = None,
+            run_cwd: T.Optional[str] = None) -> compilers.RunResult:
         extra_args = self._get_compile_extra_args(extra_args)
-        with self._build_wrapper(code, env, extra_args, dependencies, mode='link', want_output=True) as p:
-            if p.returncode != 0:
-                mlog.debug(f'Could not compile test file {p.input_name}: {p.returncode}\n')
-                return compilers.RunResult(False)
-            if need_exe_wrapper:
-                cmdlist = self.exe_wrapper.get_command() + [p.output_name]
-            else:
-                cmdlist = [p.output_name]
-            try:
-                pe, so, se = mesonlib.Popen_safe(cmdlist)
-            except Exception as e:
-                mlog.debug(f'Could not run: {cmdlist} (error: {e})\n')
-                return compilers.RunResult(False)
-
-        mlog.debug('Program stdout:\n')
-        mlog.debug(so)
-        mlog.debug('Program stderr:\n')
-        mlog.debug(se)
-        return compilers.RunResult(True, pe.returncode, so, se)
+        return super().run(code, env, extra_args, dependencies, run_env, run_cwd)
 
     def sizeof(self, typename: str, prefix: str, env: 'Environment', *,
                extra_args: T.Union[None, T.List[str], T.Callable[[CompileCheckMode], T.List[str]]] = None,
-               dependencies: T.Optional[T.List['Dependency']] = None) -> int:
+               dependencies: T.Optional[T.List['Dependency']] = None) -> T.Tuple[int, bool]:
         if extra_args is None:
             extra_args = []
         t = f'''
@@ -745,17 +597,17 @@ class DCompiler(Compiler):
             writeln(({typename}).sizeof);
         }}
         '''
-        res = self.run(t, env, extra_args=extra_args,
-                       dependencies=dependencies)
+        res = self.cached_run(t, env, extra_args=extra_args,
+                              dependencies=dependencies)
         if not res.compiled:
-            return -1
+            return -1, False
         if res.returncode != 0:
             raise mesonlib.EnvironmentException('Could not run sizeof test binary.')
-        return int(res.stdout)
+        return int(res.stdout), res.cached
 
     def alignment(self, typename: str, prefix: str, env: 'Environment', *,
                   extra_args: T.Optional[T.List[str]] = None,
-                  dependencies: T.Optional[T.List['Dependency']] = None) -> int:
+                  dependencies: T.Optional[T.List['Dependency']] = None) -> T.Tuple[int, bool]:
         if extra_args is None:
             extra_args = []
         t = f'''
@@ -774,7 +626,7 @@ class DCompiler(Compiler):
         align = int(res.stdout)
         if align == 0:
             raise mesonlib.EnvironmentException(f'Could not determine alignment of {typename}. Sorry. You might want to file a bug.')
-        return align
+        return align, res.cached
 
     def has_header(self, hname: str, prefix: str, env: 'Environment', *,
                    extra_args: T.Union[None, T.List[str], T.Callable[['CompileCheckMode'], T.List[str]]] = None,
@@ -786,7 +638,7 @@ class DCompiler(Compiler):
         import {hname};
         '''
         return self.compiles(code, env, extra_args=extra_args,
-                             dependencies=dependencies, mode='compile', disable_cache=disable_cache)
+                             dependencies=dependencies, mode=CompileCheckMode.COMPILE, disable_cache=disable_cache)
 
 class GnuDCompiler(GnuCompiler, DCompiler):
 
@@ -796,12 +648,11 @@ class GnuDCompiler(GnuCompiler, DCompiler):
 
     def __init__(self, exelist: T.List[str], version: str, for_machine: MachineChoice,
                  info: 'MachineInfo', arch: str, *,
-                 exe_wrapper: T.Optional['ExternalProgram'] = None,
                  linker: T.Optional['DynamicLinker'] = None,
                  full_version: T.Optional[str] = None,
                  is_cross: bool = False):
         DCompiler.__init__(self, exelist, version, for_machine, info, arch,
-                           exe_wrapper=exe_wrapper, linker=linker,
+                           linker=linker,
                            full_version=full_version, is_cross=is_cross)
         GnuCompiler.__init__(self, {})
         default_warn_args = ['-Wall', '-Wdeprecated']
@@ -835,8 +686,8 @@ class GnuDCompiler(GnuCompiler, DCompiler):
     def get_warn_args(self, level: str) -> T.List[str]:
         return self.warn_args[level]
 
-    def get_buildtype_args(self, buildtype: str) -> T.List[str]:
-        return d_gdc_buildtype_args[buildtype]
+    def get_optimization_args(self, optimization_level: str) -> T.List[str]:
+        return gdc_optimization_args[optimization_level]
 
     def compute_parameters_with_absolute_paths(self, parameter_list: T.List[str],
                                                build_dir: str) -> T.List[str]:
@@ -855,8 +706,10 @@ class GnuDCompiler(GnuCompiler, DCompiler):
             return args
         return args + ['-shared-libphobos']
 
-    def get_disable_assert_args(self) -> T.List[str]:
-        return ['-frelease']
+    def get_assert_args(self, disable: bool, env: 'Environment') -> T.List[str]:
+        if disable:
+            return ['-frelease']
+        return []
 
 # LDC uses the DMD frontend code to parse and analyse the code.
 # It then uses LLVM for the binary code generation and optimizations.
@@ -878,12 +731,11 @@ class LLVMDCompiler(DmdLikeCompilerMixin, DCompiler):
 
     def __init__(self, exelist: T.List[str], version: str, for_machine: MachineChoice,
                  info: 'MachineInfo', arch: str, *,
-                 exe_wrapper: T.Optional['ExternalProgram'] = None,
                  linker: T.Optional['DynamicLinker'] = None,
                  full_version: T.Optional[str] = None,
                  is_cross: bool = False, version_output: T.Optional[str] = None):
         DCompiler.__init__(self, exelist, version, for_machine, info, arch,
-                           exe_wrapper=exe_wrapper, linker=linker,
+                           linker=linker,
                            full_version=full_version, is_cross=is_cross)
         DmdLikeCompilerMixin.__init__(self, dmd_frontend_version=find_ldc_dmd_frontend_version(version_output))
         self.base_options = {OptionKey(o) for o in ['b_coverage', 'b_colorout', 'b_vscrt', 'b_ndebug']}
@@ -900,11 +752,6 @@ class LLVMDCompiler(DmdLikeCompilerMixin, DCompiler):
             return ['-wi']
         return []
 
-    def get_buildtype_args(self, buildtype: str) -> T.List[str]:
-        if buildtype != 'plain':
-            return self._get_target_arch_args() + d_ldc_buildtype_args[buildtype]
-        return d_ldc_buildtype_args[buildtype]
-
     def get_pic_args(self) -> T.List[str]:
         return ['-relocation-model=pic']
 
@@ -915,6 +762,8 @@ class LLVMDCompiler(DmdLikeCompilerMixin, DCompiler):
         return self._unix_args_to_native(args, self.info, self.linker.id)
 
     def get_optimization_args(self, optimization_level: str) -> T.List[str]:
+        if optimization_level != 'plain':
+            return self._get_target_arch_args() + ldc_optimization_args[optimization_level]
         return ldc_optimization_args[optimization_level]
 
     @classmethod
@@ -927,8 +776,10 @@ class LLVMDCompiler(DmdLikeCompilerMixin, DCompiler):
             return args
         return args + ['-link-defaultlib-shared']
 
-    def get_disable_assert_args(self) -> T.List[str]:
-        return ['--release']
+    def get_assert_args(self, disable: bool, env: 'Environment') -> T.List[str]:
+        if disable:
+            return ['--release']
+        return []
 
     def rsp_file_syntax(self) -> RSPFileSyntax:
         # We use `mesonlib.is_windows` here because we want to know what the
@@ -943,12 +794,11 @@ class DmdDCompiler(DmdLikeCompilerMixin, DCompiler):
 
     def __init__(self, exelist: T.List[str], version: str, for_machine: MachineChoice,
                  info: 'MachineInfo', arch: str, *,
-                 exe_wrapper: T.Optional['ExternalProgram'] = None,
                  linker: T.Optional['DynamicLinker'] = None,
                  full_version: T.Optional[str] = None,
                  is_cross: bool = False):
         DCompiler.__init__(self, exelist, version, for_machine, info, arch,
-                           exe_wrapper=exe_wrapper, linker=linker,
+                           linker=linker,
                            full_version=full_version, is_cross=is_cross)
         DmdLikeCompilerMixin.__init__(self, version)
         self.base_options = {OptionKey(o) for o in ['b_coverage', 'b_colorout', 'b_vscrt', 'b_ndebug']}
@@ -958,11 +808,6 @@ class DmdDCompiler(DmdLikeCompilerMixin, DCompiler):
             return ['-color=on']
         return []
 
-    def get_buildtype_args(self, buildtype: str) -> T.List[str]:
-        if buildtype != 'plain':
-            return self._get_target_arch_args() + d_dmd_buildtype_args[buildtype]
-        return d_dmd_buildtype_args[buildtype]
-
     def get_std_exe_link_args(self) -> T.List[str]:
         if self.info.is_windows():
             # DMD links against D runtime only when main symbol is found,
@@ -1004,6 +849,8 @@ class DmdDCompiler(DmdLikeCompilerMixin, DCompiler):
         return self._unix_args_to_native(args, self.info, self.linker.id)
 
     def get_optimization_args(self, optimization_level: str) -> T.List[str]:
+        if optimization_level != 'plain':
+            return self._get_target_arch_args() + dmd_optimization_args[optimization_level]
         return dmd_optimization_args[optimization_level]
 
     def can_linker_accept_rsp(self) -> bool:
@@ -1015,8 +862,10 @@ class DmdDCompiler(DmdLikeCompilerMixin, DCompiler):
             return args
         return args + ['-defaultlib=phobos2', '-debuglib=phobos2']
 
-    def get_disable_assert_args(self) -> T.List[str]:
-        return ['-release']
+    def get_assert_args(self, disable: bool, env: 'Environment') -> T.List[str]:
+        if disable:
+            return ['-release']
+        return []
 
     def rsp_file_syntax(self) -> RSPFileSyntax:
         return RSPFileSyntax.MSVC
