@@ -1,22 +1,13 @@
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2015-2016 The Meson development team
 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-
-#     http://www.apache.org/licenses/LICENSE-2.0
-
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
+from __future__ import annotations
 
 import sys, os
 import pickle, subprocess
 import typing as T
 from ..coredata import CoreData
-from ..backend.vs2010backend import RegenInfo
+from ..backend.backends import RegenInfo
 from ..mesonlib import OptionKey
 
 # This could also be used for XCode.
