@@ -1,17 +1,6 @@
+# SPDX-License-Identifier: Apache-2.0
 # Copyright 2016-2022 The Meson development team
 
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-
-#     http://www.apache.org/licenses/LICENSE-2.0
-
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-
 import stat
 import subprocess
 import re
@@ -298,43 +287,6 @@ class LinuxlikeTests(BasePlatformTests):
         self.build()
         self._run(self.mtest_command)
 
-    def test_vala_c_warnings(self):
-        '''
-        Test that no warnings are emitted for C code generated by Vala. This
-        can't be an ordinary test case because we need to inspect the compiler
-        database.
-        https://github.com/mesonbuild/meson/issues/864
-        '''
-        if not shutil.which('valac'):
-            raise SkipTest('valac not installed.')
-        testdir = os.path.join(self.vala_test_dir, '5 target glib')
-        self.init(testdir)
-        compdb = self.get_compdb()
-        vala_command = None
-        c_command = None
-        for each in compdb:
-            if each['file'].endswith('GLib.Thread.c'):
-                vala_command = each['command']
-            elif each['file'].endswith('GLib.Thread.vala'):
-                continue
-            elif each['file'].endswith('retcode.c'):
-                c_command = each['command']
-            else:
-                m = 'Unknown file {!r} in vala_c_warnings test'.format(each['file'])
-                raise AssertionError(m)
-        self.assertIsNotNone(vala_command)
-        self.assertIsNotNone(c_command)
-        # -w suppresses all warnings, should be there in Vala but not in C
-        self.assertIn(" -w ", vala_command)
-        self.assertNotIn(" -w ", c_command)
-        # -Wall enables all warnings, should be there in C but not in Vala
-        self.assertNotIn(" -Wall ", vala_command)
-        self.assertIn(" -Wall ", c_command)
-        # -Werror converts warnings to errors, should always be there since it's
-        # injected by an unrelated piece of code and the project has werror=true
-        self.assertIn(" -Werror ", vala_command)
-        self.assertIn(" -Werror ", c_command)
-
     @skipIfNoPkgconfig
     def test_qtdependency_pkgconfig_detection(self):
         '''
@@ -976,6 +928,7 @@ class LinuxlikeTests(BasePlatformTests):
         install_rpath = get_rpath(os.path.join(self.installdir, 'usr/bin/progcxx'))
         self.assertEqual(install_rpath, 'baz:/foo/dummy')
 
+    @skipIfNoPkgconfig
     def test_global_rpath(self):
         if is_cygwin():
             raise SkipTest('Windows PE/COFF binaries do not use RPATH')
@@ -1171,7 +1124,7 @@ class LinuxlikeTests(BasePlatformTests):
         # option, adding the meson-uninstalled directory to it.
         PkgConfigInterface.setup_env({}, env, MachineChoice.HOST, uninstalled=True)
 
-        pkg_config_path = env.coredata.options[OptionKey('pkg_config_path')].value
+        pkg_config_path = env.coredata.optstore.get_value('pkg_config_path')
         self.assertEqual(pkg_config_path, [pkg_dir])
 
     @skipIfNoPkgconfig
@@ -1458,7 +1411,7 @@ class LinuxlikeTests(BasePlatformTests):
         Test that installation of broken symlinks works fine.
         https://github.com/mesonbuild/meson/issues/3914
         '''
-        testdir = os.path.join(self.common_test_dir, testdir)
+        testdir = self.copy_srcdir(os.path.join(self.common_test_dir, testdir))
         subdir = os.path.join(testdir, subdir_path)
         with chdir(subdir):
             # Can't distribute broken symlinks in the source tree because it breaks
@@ -1466,17 +1419,14 @@ class LinuxlikeTests(BasePlatformTests):
             # hand.
             src = '../../nonexistent.txt'
             os.symlink(src, 'invalid-symlink.txt')
-            try:
-                self.init(testdir)
-                self.build()
-                self.install()
-                install_path = subdir_path.split(os.path.sep)[-1]
-                link = os.path.join(self.installdir, 'usr', 'share', install_path, 'invalid-symlink.txt')
-                self.assertTrue(os.path.islink(link), msg=link)
-                self.assertEqual(src, os.readlink(link))
-                self.assertFalse(os.path.isfile(link), msg=link)
-            finally:
-                os.remove(os.path.join(subdir, 'invalid-symlink.txt'))
+            self.init(testdir)
+            self.build()
+            self.install()
+            install_path = subdir_path.split(os.path.sep)[-1]
+            link = os.path.join(self.installdir, 'usr', 'share', install_path, 'invalid-symlink.txt')
+            self.assertTrue(os.path.islink(link), msg=link)
+            self.assertEqual(src, os.readlink(link))
+            self.assertFalse(os.path.isfile(link), msg=link)
 
     def test_install_subdir_symlinks(self):
         self.install_subdir_invalid_symlinks('59 install subdir', os.path.join('sub', 'sub1'))
